// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
#include <string>
using namespace std;

class BadFilename { };
class FileNotFound { };

bool isValidFilename(const string& filename) throw()
{
  // Pretend this checks if filename is a valid filename
  return true;
}

bool fileExists(const string& filename) throw()
{
  // Pretend this checks if filename exists as a file
  return false;
}

void processFile(const string& filename) throw(BadFilename, FileNotFound)
{
  if (! isValidFilename(filename))
    throw BadFilename();

  if (! fileExists(filename))
    throw FileNotFound();

  // the filename is valid and exists; process the file:
  // ...
}

void f(const string& filename) throw()
{
  try {
    processFile(filename);
    // ...
  }
  catch (BadFilename& e) {
    cout << "Invalid file name: " << filename << "\n";
  }
  catch (FileNotFound& e) {
    cout << "File not found: " << filename << "\n";
  }
}

int main()
{
  try {
    f("input-file.txt");
    // ...
  }
  catch (...) {
    cout << "Unknown exception!\n";
  }
}
