// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Bird {
public:
  Bird() throw();
  virtual ~Bird() throw();
  int altitude() const throw();
  virtual void fly() throw();
    // PROMISE: altitude() will return a number > 0; never throws an exception.
protected:
  int altitude_;
};

Bird::Bird() throw()
: altitude_(0)
{ }

Bird::~Bird() throw()
{ }

int Bird::altitude() const throw()
{ return altitude_; }

void Bird::fly() throw()
{ altitude_ = 100; }

void sample(Bird& bird) throw()  // Legitimate reliance on what Bird::fly() says
{
  bird.fly();
}

class CannotFly { };

class Ostrich : public Bird {
public:
  virtual void fly() throw(CannotFly);
    // PROMISE: throws an exception despite what Bird says
};

void Ostrich::fly() throw(CannotFly)
{ throw CannotFly(); }

int main()
{
  Ostrich bird;
  sample(bird);  // Legitimate conversion from Ostrich to Bird
}
