// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <cassert>
#include <cstdlib>
#include <stdexcept>
#include <string>
using namespace std;

class List;

class Node {
private:
  friend List;
  int   elem_;
  Node* next_;
  Node(int elem, Node* next) throw();
};

Node::Node(int elem, Node* next) throw()
: elem_(elem)
, next_(next)
{ }

class Empty : public runtime_error {
public:
  Empty(const string& what);
};

Empty::Empty(const string& what)
: runtime_error(what)
{ }

class List {
public:
  List() throw();
  bool empty() const throw();
  int  size() const throw();
  int  peekAtFirst() const throw(Empty);
  void prepend(int x) throw();
  int  removeFirst() throw(Empty);
    // REQUIRE: empty() must return false
    // PROMISE: return value will be the initial value of peekAtFirst().
    // PROMISE: size() will be reduced by 1.
  List            (const List& list) throw();  // copy constructor; not shown
  List& operator= (const List& list) throw();  // assignment operator; not shown
  ~List() throw();
protected:
  Node* first_;
};

List::List() throw()
: first_(NULL)
{ }

List::~List() throw()
{ while (first_ != NULL) removeFirst(); }

bool List::empty() const throw()
{ return first_ == NULL; }

int List::size() const throw()
{
  int ans = 0;
  for (const Node* p = first_; p != NULL; p = p->next_)
    ++ans;
  return ans;
}

int List::peekAtFirst() const throw(Empty)
{
  if (empty()) throw Empty("List::peekAtFirst()");
  return first_->elem_;
}

void List::prepend(int x) throw()
{ first_ = new Node(x, first_); }

int List::removeFirst() throw(Empty)
{
  if (empty()) throw Empty("List::removeFirst()");

  #ifndef NDEBUG
    int peekAtFirstInit = peekAtFirst();
    int sizeInit = size();
  #endif

  // remove first element from the List
  int result = first_->elem_;
  Node* oldFirstNode = first_;
  first_ = first_->next_;
  delete oldFirstNode;

  #ifndef NDEBUG
    assert(result == peekAtFirstInit);
    assert(size() == sizeInit - 1);
  #endif

  return result;
}

int main()
{
  List a;
  a.prepend(42);
  a.prepend(24);
  int elem = a.removeFirst();
}
