// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <cassert>

class Date {
public:
  Date() throw();
  Date& operator++ () throw();
  int month() const throw();
  int day()   const throw();
  int year()  const throw();
protected:
  void testInvariant() const throw();
  int month_;
  int day_;
  int year_;
};

inline void Date::testInvariant() const throw()
{
  assert(day_ >= 1 && day_ <= 31);
  assert(month_ >= 1 && month_ <= 12);
}

Date::Date() throw()
: month_(7)
, day_  (4)
, year_ (1776)
{
  testInvariant();  // public: constructors must establish the invariant
}

int Date::month() const throw()
{
  testInvariant();  // Wild pointers can corrupt a sleeping object
  return month_;
}

int Date::day() const throw()
{
  testInvariant();  // Wild pointers can corrupt a sleeping object
  return day_;
}

int Date::year() const throw()
{
  testInvariant();  // Wild pointers can corrupt a sleeping object
  return year_;
}

Date& Date::operator++ () throw()
{
  testInvariant();  // Wild pointers can corrupt a sleeping object
  // ...            // Put code to increment the date here
  testInvariant();  // public: mutators must maintain the invariant
  return *this;
}

