// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#ifdef BROKEN_NEW_HANDLER
#include <new.h>
#endif

#include <new>
using namespace std;

class Fred {
public:
  static Fred* create() throw(bad_alloc); // Named Constructor Idiom; see FAQ 16.08
  virtual void discard() throw();    // p->discard() is analogous to delete p
  static bool flushPool() throw();   // Returns true if it freed some memory
private:
  Fred()  throw();      // Constructor is private: so users can't say new Fred()
  ~Fred() throw();      // Destructor is private: so users can't say delete p
  void init() throw();  // Initializes (or possibly re-initializes) the object as if it was just created
  Fred* nextRecycled_;
  static Fred* headRecycled_;
};

void Fred::init() throw()
{
  // ...        // This is where the constructor's logic should go
  nextRecycled_ = NULL;
}

Fred::Fred()
{ init(); }

Fred::~Fred()
{ }             // The destructor doesn't need to do anything with the recycled list

Fred* Fred::headRecycled_ = NULL;

Fred* Fred::create() throw(bad_alloc)
{
  if (headRecycled_ == NULL)
    return new Fred();
  Fred* p = headRecycled_;
  headRecycled_ = headRecycled_->nextRecycled_;
  p->init();    // Re-initializes the object as if it was just created
  return p;
}

void Fred::discard() throw()
{
  nextRecycled_ = headRecycled_;
  headRecycled_ = this;
}

bool Fred::flushPool() throw()
{
  bool stuffGotDeleted = (headRecycled_ != NULL);
  while (headRecycled_ != NULL)
    delete create();
  return stuffGotDeleted;
}

#include <new>
using namespace std;

void flushAllPools() throw(bad_alloc)
{
  unsigned n = 0;
  n += Fred::flushPool();
  // Flush any other pools as well; e.g., n += Wilma::flushPool();
  if (n == 0) throw bad_alloc();  // Nobody freed memory; prevent infinite loop
}

#ifdef BROKEN_NEW_HANDLER
int flushAllPools(unsigned) throw(bad_alloc)
{ flushAllPools(); return 0; }
#endif

int main()
{
#ifdef BROKEN_NEW_HANDLER
  _set_new_handler(flushAllPools);  // Install the "new handler"
#else
  set_new_handler(flushAllPools);   // Install the "new handler"
#endif
  // ...
}
