// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Car {
public:

  Car() throw();              // Increments Car::num_
  Car(const Car& c) throw();  // Increments Car::num_

  // No need for an explicit assignment operator or destructor
  // since these don't create new Car objects.
  // (see FAQ [[the big 3]]).

  static int num_;   // Class data

private:
  int serial_;       // Object data
};

Car::Car() throw()
: serial_(num_)
{
  cout << "Car ctor\n";
  ++num_;
}

Car::Car(const Car& c) throw()
: serial_(num_)
{
  cout << "Car copy\n";
  ++num_;
}

int Car::num_ = 0;   // Class data is automatically initialized, often before main()

int main()
{
  cout << "Car::num_ = " << Car::num_ << '\n';
  {
    Car a;
    cout << "Car::num_ = " << Car::num_ << '\n';
    Car b;
    cout << "Car::num_ = " << Car::num_ << '\n';
    Car c = a;
    cout << "Car::num_ = " << Car::num_ << '\n';
  }
  cout << "Car::num_ = " << Car::num_ << '\n';
}
