// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
#include <stdexcept>
using namespace std;

int openFile(const char* name) throw()
{
  // normally this code would actually open the named file.
  // for this example, we pretend everything is handle 42.
  int handle = 42;
  cout << "opening " << name << " as #" << handle << '\n';
  return handle;
}

void closeFile(int handle) throw()
{
  // normally this code would actually close the file.
  cout << "closing file #" << handle << '\n';
}

class File {
public:
  File(const char* name) throw();
  ~File() throw();
  void close() throw();
protected:
  enum { closed_ = -1 };  // File::closed_ is a constant
  int handle_;
private:
  // these are never defined; copy semantics are ill defined
  File(const File&);
  File& operator= (const File&);
};

inline void File::close()
{
  if (handle_ != closed_) closeFile(handle_);
  handle_ = closed_;
}

inline File::File(const char* name) throw()
: handle_( openFile(name) ) { }

inline File::~File() throw()
{ close(); }

void userCode(bool throwIt) throw(runtime_error)
{
  File x("sample.txt");
  cout << "after open, before throw-or-close\n";
  if (throwIt) throw runtime_error("note that the file still gets closed!");
  x.close();
  cout << "after close\n";
}

int main()
{
  cout << "====== without throwing an exception ======\n";
  userCode(false);

  cout << "====== with throwing an exception =========\n";
  try {
    userCode(true);
  }
  catch (exception& e) {
    cout << "exception caught; " << e.what() << '\n';
  }
}
