// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

class Stack {
public:
  virtual ~Stack()                    throw();
  virtual void   push(int elem)       throw() = 0;
  virtual int    pop()                throw() = 0;
  virtual int    getElem(int n) const throw() = 0;
  virtual Stack& operator= (const Stack& s) throw();
protected:
  int n_;
};

Stack::~Stack() throw()
{ }

Stack& Stack::operator= (const Stack& s) throw()
{ n_ = s.n_; return *this; }

void userCode(Stack& s, Stack& s2)
{ s = s2; }

class StackArray : public Stack {
public:
  StackArray()                       throw();
  virtual void push(int x)           throw();
  virtual int  pop()                 throw();
  virtual int  getElem(int n) const  throw();
#ifdef COVARIANT_RETURN_TYPE_BUG
  virtual Stack& operator= (const Stack& s) throw();  //override
#else
  virtual StackArray& operator= (const Stack& s) throw();  //override
#endif
  StackArray& operator= (const StackArray& s)    throw();  //overload
protected:
  int data_[10];
};

StackArray::StackArray()             throw() : Stack() { }
void StackArray::push(int x)         throw() { data_[n_++] = x; }
int StackArray::pop()                throw() { return data_[--n_]; }
int StackArray::getElem(int n) const throw() { return data_[n]; }

// Override:
#ifdef COVARIANT_RETURN_TYPE_BUG
Stack& StackArray::operator= (const Stack& s) throw()
#else
StackArray& StackArray::operator= (const Stack& s) throw()
#endif
{
  Stack::operator= (s);
  for (int i = 0; i < n_; ++i)
    data_[i] = s.getElem(i);
  return *this;
}

// Overload:
StackArray& StackArray::operator= (const StackArray& s) throw()
{
  Stack::operator= (s);
  for (int i = 0; i < n_; ++i)
    data_[i] = s.data_[i];
  return *this;
}

int main()
{
  StackArray s, s2;
  userCode(s, s2);
}
