// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class B {
public:
  virtual ~B() throw();
  virtual B& operator= (const B& b) throw();
};

B::~B() throw()
{ }

B& B::operator= (const B& b) throw()
{ cout << "B::operator=(const B&)\n"; return *this; }

class D : public B {
public:
#ifdef COVARIANT_RETURN_TYPE_BUG
  virtual B& operator= (const B& b) throw();  // override
#else
  virtual D& operator= (const B& b) throw();  // override
#endif
  D& operator= (const D& d) throw();          // overload
};

#ifdef COVARIANT_RETURN_TYPE_BUG
B& D::operator= (const B& b) throw()  // override
#else
D& D::operator= (const B& b) throw()  // override
#endif
{ cout << "D::operator=(const B&)\n"; return *this; }

D& D::operator= (const D& d) throw()  // overload
{ cout << "D::operator=(const D&)\n"; return *this; }

void sample(D& d, B& b, D& d2, B& b2) throw()
{
  cout << "d = d2:  ";  d = d2;
  cout << "d = b2:  ";  d = b2;
  cout << "b = b2:  ";  b = b2;
  cout << "b = d2:  ";  b = d2;
}

int main()
{
  D d, b, d2, b2;
  sample(d, b, d2, b2);
}

