// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"
#include <stdexcept>
using namespace std;

template<class T>    // T is a type in the declaration that follows
class Array {
public:
  Array(unsigned size=10);
  Array(const Array<T>& a);                //copy constructor
  Array<T>& operator= (const Array<T>& a); //assignment
 ~Array() throw();
  unsigned size() const throw();
  const T& operator[] (unsigned i) const throw(out_of_range);
  T&       operator[] (unsigned i)       throw(out_of_range);
protected:
  unsigned  size_;
  T*        arr_;
};

template<class T> inline Array<T>::Array(unsigned size)
: size_(size)
, arr_(new T[size])
{ }

template<class T> inline Array<T>::~Array() throw()
{ delete[] arr_;  }

template<class T> inline unsigned Array<T>::size() const throw()
{ return size_; }

template<class T> inline const T& Array<T>::operator[] (unsigned i) const
throw(out_of_range)
{
  if (i >= size_) throw out_of_range("Array<T>::operator[]"));
  return arr_[i];
}

template<class T> inline T& Array<T>::operator[] (unsigned i)
throw(out_of_range)
{
  if (i >= size_) throw out_of_range("Array<T>::operator[] const");
  return arr_[i];
}

#include <string>
//#include "Array.hpp"
using namespace std;

int main()
{
  Array<int>    ai;  ai[5] = 42;
  Array<float>  af;  af[5] = 42.0;
  Array<char>   ac;  ac[5] = 'x';
  Array<string> as;  as[5] = "xyz";
}
