// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <memory>
using namespace std;

template<class T>
class strict_auto_ptr : public auto_ptr<T> {
public:
  strict_auto_ptr(T* p=NULL) throw() : auto_ptr<T>(p) { }
private:
  strict_auto_ptr(const strict_auto_ptr&) throw();
  void operator= (const strict_auto_ptr&) throw();
};

#include <iostream>
using namespace std;

class Noisy {
public:
  Noisy() throw();
 ~Noisy() throw();
  Noisy& operator= (const Noisy&) throw();
  Noisy            (const Noisy&) throw();
};

Noisy::Noisy() throw()
  { cout << "Noisy::Noisy()\n"; }
Noisy::~Noisy() throw()
  { cout << "Noisy::~Noisy()\n"; }

typedef strict_auto_ptr<Noisy> NoisyPtr;

class Fred {
public:
  Fred() throw(bad_alloc);
  //No destructor needed: The Noisy will automagically get
  //deleted. The compiler won't synthesize a copy ctor or
  //assignment operator, since the strict_auto_ptr version of these
  //are private.
protected:
  NoisyPtr ptr_;  // like Noisy* ptr_ but better
};

Fred::Fred() throw(bad_alloc)
: ptr_(new Noisy())
{ }

void sample()
{
  Fred x;  //OK: allocates a new Noisy
}          //OK: x is destructed, so its Noisy gets deleted

int main()
{ sample(); }

void disasterAverted(const Fred& x) throw()
{
  #ifdef GENERATE_ERROR
    Fred y = x;  //gives a compile-time error message
    y = x;       //gives a compile-time error message
  #endif
}
