// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <stdexcept>
using namespace std;

class Fred {
public:
  Fred() throw(); // A default constructor
  // ...
private:
  // ...
  class Data {
  public:
    Data() throw();
    Data(int i, int j) throw();
    Data(const Data& d) throw();

    // Since only Fred can access Fred::Data,
    // the Fred::Data members can be public:.
    // If that feels uncomfortable, the members can be made private:
    // and class Fred can be made a friend class.
    // See FAQ 19-01
    // ...

    unsigned count_;
    // count_ is the number of Fred objects that point at this
    // count_ must be initialized to 1 by all constructors
    // (starts at 1 because of the Fred object that created this)
  };

  Data* data_;
  static Data* defaultData() throw();
};

Fred::Fred() throw()
: data_(defaultData())
{ ++ data_->count_; }

Fred::Data* Fred::defaultData() throw(bad_alloc)
{
  static Data* p = NULL;
  if (p == NULL) {
    p = new Data();
    ++ p->count_;   // Make sure it never goes to zero
  }
  return p;
}
