// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
#include <memory>
using namespace std;

class Base {
public:
  virtual ~Base() throw();
  virtual void f() throw() = 0;
};

typedef auto_ptr<Base> BasePtr;

Base::~Base() throw()
{ }

class Derived : public Base {
public:
  virtual void f() throw();
};

void Derived::f() throw()
{ cout << "Derived::f()\n"; }

class Fred {
public:
  Fred() throw(bad_alloc);
 ~Fred() throw();
  Fred& operator= (const Fred& a) throw();
  Fred(const Fred& a) throw();
  void method() throw();
  void blahBlah() throw(bad_alloc);
private:
  BasePtr x_;  // has-a via remote-ownership; see FAQ 30.08
  Derived y_;  // has-a via direct object containment
};

Fred::Fred() throw(bad_alloc)
: x_(new Derived())
, y_()
{ }

void Fred::method() throw()
{
  x_->f();  // may or may not invoke Derived::f() depending on the exact class of *x_
  y_.f();   // cannot be changed at run-time; always calls Derived::f()
}

class Derived2 : public Base {
public:
  virtual void f() throw();
};

void Derived2::f() throw()
{ cout << "Derived2::f()\n"; }

void Fred::blahBlah() throw(bad_alloc)
{
  BasePtr x2( new Derived2() );
  x_ = x2;       // Safe: automagically deletes the object formerly pointed to by x_
}                // Safe: x2 will be NULL so this doesn't delete the new object

int main()
{
  Fred a;
  a.method();    // a.x_ will point to a Derived
  a.blahBlah();  // changes a.x_
  a.method();    // a.x_ will point to a Derived2
}
