// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <memory>
using namespace std;

class Base {
public:
  virtual ~Base() throw();
  virtual void f() throw() = 0;
};
typedef auto_ptr<Base> BasePtr;

Base::~Base() throw()
{ }

class Derived : public Base {
public:
  virtual void f() throw();
};

inline void Derived::f() throw()  // This is inline even though it's virtual
{
  // If this is short, the inlining can make a significant difference
}

class Fred {
public:
  Fred() throw();
  virtual void sample() throw();
  virtual void sample2() throw();
protected:
  Derived d_;
  BasePtr b_;
};

Fred::Fred() throw()
: d_()
, b_(new Derived())
{ }

inline void Fred::sample() throw()
{ d_.f(); }  // The call to d_.f() can be inlined

inline void Fred::sample2() throw()
{ b_->f(); }  // The call to b_->f() cannot be inlined

int main()
{
  Fred x;
  x.sample();
  x.sample2();
}
