// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "wtypes.h"

extern HRESULT STACK_E_UNDERFLOW;

/* {FC3B3F61-BCEC-11D1-91FE-E1CBED988F66} */
DEFINE_GUID(IID_IStack,
    0xFC3B3F61, 0xBCEC, 0x11D1, 0x91, 0xFE,
    0xE1, 0xCB, 0xED, 0x98, 0x8F, 0x66);

DECLARE_INTERFACE_(IStack, IUnknown)
{
    // *** IStack methods *** //
    STDMETHOD(Push)  (THIS_ long  value) PURE;
    STDMETHOD(Pop)   (THIS_ long* value) PURE;
    STDMETHOD(Empty) (THIS_ long* flag) PURE;
};

// {FC3B3F62-BCEC-11D1-91FE-E1CBED988F66}
DEFINE_GUID(CLSID_CoStack,
    0xFC3B3F62, 0xBCEC, 0x11D1, 0x91, 0xFE,
    0xE1, 0xCB, 0xED, 0x98, 0x8F, 0x66);

#include <stack>
using namespace std;

class CoStack : public IStack {
public:
    // *** IUnknown methods *** //
    STDMETHOD(QueryInterface) (REFIID riid, void** ppv);
    STDMETHOD_(ULONG,AddRef) ();
    STDMETHOD_(ULONG,Release) ();

    // *** IStack methods *** //
    STDMETHOD(Push) (long  value);
    STDMETHOD(Pop)  (long* value);
    STDMETHOD(Empty)(long* flag);

    CoStack();
private:
    ULONG refCnt_;
    stack<long> data_;
};

CoStack::CoStack()
: refCnt_(0)
, data_()
{ }

STDMETHODIMP CoStack::QueryInterface(REFIID riid, void** ppv)
{
    if (riid == IID_IUnknown)
        *ppv = this;
    else if (riid == IID_IStack)
        *ppv = this;
    else {
        *ppv = NULL;
        return E_NOINTERFACE;
    }
    AddRef();
    return S_OK;
}

STDMETHODIMP_(ULONG) CoStack::AddRef()
{ return ++refCnt_; }

STDMETHODIMP_(ULONG) CoStack::Release()
{
    ULONG result = --refCnt_;
    if (result == 0)
        delete this;
    return result;
}
STDMETHODIMP CoStack::Push(long value)
{
    data_.push(value);
    return S_OK;
}

STDMETHODIMP CoStack::Pop(long* value)
{
    if (data_.empty())
        return STACK_E_UNDERFLOW;
    *value = data_.top();
    data_.pop();
    return S_OK;
}

STDMETHODIMP CoStack::Empty(long* flag)
{
    *flag = data_.empty() ? 1 : 0;
    return S_OK;
}

class CoStackClassObject : public IClassFactory {
public:
    STDMETHOD(QueryInterface) (REFIID riid, void** ppv);
    STDMETHOD_(ULONG,AddRef) ();
    STDMETHOD_(ULONG,Release) ();
    STDMETHOD(CreateInstance) (IUnknown* outer, REFIID riid, void** ppv);
    STDMETHOD(LockServer) (BOOL b);
};


STDMETHODIMP CoStackClassObject::QueryInterface(REFIID riid, void** ppv)
{
    if (riid == IID_IUnknown || riid == IID_IClassFactory)
        *ppv = this;
    else {
        *ppv = NULL;
        return E_NOINTERFACE;
    }
    AddRef();
    return S_OK;
}

STDMETHODIMP_(ULONG) CoStackClassObject::AddRef() { /*...*/ }

STDMETHODIMP_(ULONG) CoStackClassObject::Release() { /*...*/ }

STDMETHODIMP CoStackClassObject::CreateInstance
                 (IUnknown* outer, REFIID riid, void** ppv)
{
    *ppv = NULL;
    if (outer != NULL)
        return CLASS_E_NOAGGREGATION;
    CoStack* p = new CoStack;
    if (p == NULL)
        return E_OUTOFMEMORY;
    p->AddRef();
    HRESULT hr = p->QueryInterface(riid, ppv);
    p->Release();
    return hr;
}

STDMETHODIMP CoStackClassObject::LockServer(BOOL b) { /*...*/ }
